angular.module('configure', ['angulartics', 'angulartics.google.analytics'])

	.controller('ConfigCtrl', ['$scope', '$http', 'Page', '$analytics', '$routeParams', '$cookieStore',
				function($scope, $http, Page, $analytics, $routeParams, $cookieStore) {
								
		Page.setTitle('Configure');

		$scope.tabs = [];
		$scope.currentTabIndex = 0;
		$scope.slugs = ["appliances", "protected-assets", "copied-assets"];

		$scope.selectedStorage = null;
		if ($routeParams.id != undefined) {
		    // If route parameters are set, this was called with a pre-selected storage object.
            // For cold backup copy (archive) storage, prepend the id with 'a' as this is what the API returns that
            // populates the Backup Copy Targets tab, to differentiate archive targets from replication targets.
			$scope.selectedStorage = {};
			$scope.selectedStorage.usage = $routeParams.usage;
			$scope.selectedStorage.id = $scope.selectedStorage.usage == 'archive' ? 'a' + $routeParams.id : $routeParams.id;
			$scope.selectedStorage.sid = $routeParams.sid;
		}

		$scope.configPageLoaded = false;
		$scope.$on('$includeContentLoaded', function(event, url) {
			// Only process configure page once.  Otherwise, it is processed for EVERY included file.
			// We could look for first URL loaded, but in case it changes, instead we use a global flag to load once.
			if (!$scope.configPageLoaded) {
				$scope.configPageLoaded = true;
				processConfigurePage(PHD, window, $analytics);
			}
		});

        /*
         * Called when appliances page is loaded.  If storage was pre-selected, queue the selection of tha storage on tab 0 (appliances).
         */
		$scope.applianceLoaded = function() {
			$scope.tabs[0] = new PHD.ApplianceTab(0, $("#config-appliances")).init();
			if ($scope.currentTabIndex == 0) {
				$scope.tabs[0].activate();
				$(document).trigger("tabchange", "config" + $scope.slugs[$scope.currentTabIndex]);
				if ($scope.selectedStorage != null) {
					var selected = $scope.selectedStorage;
					$scope.selectedStorage = null;
					setTimeout(function () {
						$scope.tabs[0].queueSelection(selected);
					}, 0);
				}
			}
		};

        /*
         * Called when assets page is loaded.  If storage was pre-selected, change to tab 0 (appliances) and queue the selection of tha storage.
         */
		$scope.assetsLoaded = function() {
			$scope.tabs[1] = new PHD.AssetsTab(1, $("#config-assets")).init();
			if ($scope.currentTabIndex == 1) {
				if ($scope.selectedStorage != null) {
					$('#configure-page-tabs.tabs-wrapper').tabs("option", "active", 0);
					var selected = $scope.selectedStorage;
					$scope.selectedStorage = null;
					setTimeout(function () {
						$scope.tabs[0].queueSelection(selected);
					}, 0);
				} else {
					$scope.tabs[1].activate();
					$(document).trigger("tabchange", "config" + $scope.slugs[$scope.currentTabIndex]);
				}
			}
		};

        /*
         * Called when copied assets page is loaded.  If storage was pre-selected, change to tab 0 (appliances) and queue the selection of tha storage.
         */
		$scope.copiedAssetsLoaded = function() {
			$scope.tabs[2] = new PHD.CopiedAssetsTab(2, $("#config-copied-assets")).init();
			if ($scope.currentTabIndex == 2) {
				if ($scope.selectedStorage != null) {
					$('#configure-page-tabs.tabs-wrapper').tabs("option", "active", 0);
					var selected = $scope.selectedStorage;
					$scope.selectedStorage = null;
					setTimeout(function () {
						$scope.tabs[0].queueSelection(selected);
					}, 0);
				} else {
					$scope.tabs[2].activate();
					$(document).trigger("tabchange", "config" + $scope.slugs[$scope.currentTabIndex]);
				}
			}
		};

		// wrap existing jQuery functionality.
		function processConfigurePage(PHD, window, $analytics) {
			var initialTab = $cookieStore.get("initial_tab_config") || 0,
				$emailEditForm,
				$hvRemoveForm = $("#hypervisor-remove-form"),
				$applianceTable = $("#appliance-table"),
				hypervisorFormEdit = false,
				REFRESH_INTERVAL = 6000;

			function initTabs() {
				$scope.tabs[0] = {}; //new PHD.ApplianceTab(0, $(".ui-tabs-panel").eq(0)).init();
				$scope.tabs[1] = {}; // new PHD.ApplianceTab(1, $(".ui-tabs-panel").eq(1)).init();
				$scope.tabs[2] = {}; //new PHD.CopiedAssetsTab(2, $(".ui-tabs-panel").eq(2)).init();
				//tabs[3] = new PHD.LicenseTab(3, $(".ui-tabs-panel").eq(3)).init();

				//$scope.tabs[currentTabIndex].activate();
				//$(document).trigger("tabchange", "config" + slugs[currentTabIndex]);
			}

			function initForm() {
				/**
				 * Return true if the field value matches the given format RegExp
				 *
				 * @example jQuery.validator.methods.pattern("AR1004",element,/^AR\d{4}$/)
				 * @result true
				 *
				 * @example jQuery.validator.methods.pattern("BR1004",element,/^AR\d{4}$/)
				 * @result false
				 *
				 * @name jQuery.validator.methods.pattern
				 * @type Boolean
				 * @cat Plugins/Validate/Methods
				 */
				$.validator.addMethod("pattern", function (value, element, param) {
					if (this.optional(element)) {
						return true;
					}
					if (typeof param === 'string') {
						param = new RegExp('^(?:' + param + ')$');
					}
					return param.test(value);
				}, gettext("Invalid format."));
			}

			/*
			 *   DOM Ready Function
			 */
			$(function () {
				// Initialize form validation and event handlers
				initForm();

				// Quick and dirty way to change what tab opens by default via URL
				if (!isNaN(PHD.util.getHashInt())) {
					initialTab = PHD.util.getHashInt();
				}

				// Initialize the tabs
				$('#configure-page-tabs.tabs-wrapper').tabs({
					active: initialTab,
					create: function (event, ui) {
						$scope.currentTabIndex = initialTab; // lmc - using angular cookie store, and loading once, was: ui.tab.index();
						initTabs();
					},
					activate: function (event, ui) {
						$(".loading").hide();
						$scope.currentTabIndex = ui.newTab.index();
						$cookieStore.put("initial_tab_config", $scope.currentTabIndex, {expires: 365, path: '/'});
						$scope.tabs[ui.oldTab.index()].deactivate();
						$scope.tabs[$scope.currentTabIndex].activate();
						$(document).trigger("tabchange", "config" + $scope.slugs[$scope.currentTabIndex]);
					}
				});
			});

		}

	}]
);